from abc import ABC, abstractmethod
from typing import Any


class AbstractStorageClient(ABC):
    @abstractmethod
    def retrieve(self, key: dict[Any, Any], **call_params: Any):
        ...

    @abstractmethod
    def create(self, table_item: dict[Any, Any], **call_params: Any):
        ...

    @abstractmethod
    def update(self, key: dict[Any, Any], attribute_updates: dict[Any, Any], **call_params: Any):
        ...

    @abstractmethod
    def delete(self, key: Any, **call_params: Any):
        ...

    @abstractmethod
    def bulk_retrieve(self, keys: list[dict[Any, Any]], **call_params: Any):
        ...

    @abstractmethod
    def bulk_create(self, table_items: list[dict[Any, Any]], **call_params: Any):
        ...

    @abstractmethod
    def bulk_update(self, keys: list[dict[Any, Any]], attribute_updates_list: list[dict[Any, Any]], **call_params: Any):
        ...

    @abstractmethod
    def bulk_delete(self, keys: list[dict[Any, Any]], **call_params: Any):
        ...



from typing import Any
from uuid import UUID, uuid4

from pydantic import Field, HttpUrl, field_validator

from rooms_shared_services.src.models.texts.languages import Language
from rooms_shared_services.src.models.texts.translations import TextTranslations
from rooms_shared_services.src.storage.models import BaseDynamodbModel


class ProductCategory(BaseDynamodbModel):
    name: str
    id: int
    parent_id: int | None = None
    parent_name: str | None = None
    description: str | None = None
    image: HttpUrl | None = None
    name_translations: TextTranslations | None = None
    description_translations: TextTranslations | None = None

    @field_validator(
        "image",
        mode="before",
    )
    def coerce_image(cls, item_value: Any):
        if isinstance(item_value, dict):
            return item_value.get("image", None)
        return item_value


class ProductCategoryTranslation(BaseDynamodbModel):
    id: UUID = Field(default_factory=uuid4)
    cat_name: str
    cat_language: Language
    cat_description: str | None = None
    cat_image: HttpUrl | None = None

    @field_validator("cat_name", mode="before")
    def remove_underscores(cls, name_value: Any):
        if isinstance(name_value, str):
            return name_value.replace("_", " ").lower()
        return name_value

    @field_validator("cat_language", mode="before")
    def coerce_language(cls, language_name: Any):
        if isinstance(language_name, str):
            try:
                return getattr(Language, language_name)
            except AttributeError:
                pass
        return language_name



from typing import Literal

from pydantic_settings import BaseSettings, SettingsConfigDict

LevelLiteral = Literal["DEBUG", "INFO", "WARN", "ERROR"]


class Settings(BaseSettings):
    """HTTP logging settings."""

    model_config = SettingsConfigDict(env_prefix="http_logging_")

    host: str
    path: str
    secure: bool = True
    loglevel: LevelLiteral = "INFO"
